#! python

# #####################################################################
# Copyright (c) 2017, Stanley Engle
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# #####################################################################

# Standard library imports.
import argparse
import sys


# The version number of this script/module.
__version__ = '2'


def calculate(tmax_c, tmin_c, on_error = float('nan'), **kwargs):
    """Calculate the growing degree-days.

    The growing degree-days are calculated using the maximum and 
    minimum daily air temperatures in degrees Celsius.  See:
    https://en.wikipedia.org/wiki/Growing_degree-day

    Parameters
    ----------
    tmax_c : float
        The maximum daily air temperature in degrees C.
    tmin_c : float
        The minimum daily air temperature in degrees C.
    on_error : optional, float('nan')
        The return value when an error occcurs during calculation.  May 
        be any type, but preferably a float.
    tbase : optional, 10.0
        The base temperature, in degrees C.
    tmaxcutoff : optional, 999.0
        Sets the maximum value for the 'tmax_c' parameter.
    tmincutoff : optional, -999.0
        Sets the minimum value for the 'tmin_c" parameter.

    Returns
    -------
    float
        The growing degree-days calculated using Celsius values.  To 
        convert, multiple the return value by (9.0 / 5.0).

    """
    tbase = kwargs.get('tbase', 10.0)
    tmaxcutoff = kwargs.get('tmaxcutoff', 999.0)
    tmincutoff = kwargs.get('tmincutoff', -999.0)

    try:

        _tmax = float(tmax_c)
        _tmin = float(tmin_c)

        _tbase = float(tbase)
        _tmaxcutoff = float(tmaxcutoff)
        _tmincutoff = float(tmincutoff)

        if _tmax > _tmaxcutoff:
            _tmax = _tmaxcutoff

        if _tmin < _tmincutoff:
            _tmin = _tmincutoff

        gdd = ((_tmax + _tmin) / 2.0) - _tbase

        if gdd < 0.0:
            gdd = 0.0

    except:
        gdd = on_error

    return gdd


def run(tmax_c, tmin_c, on_error = float('nan'), **kwargs):
    """Calls the calculate function.  See calculate docstring.

    """
    return calculate(tmax_c, tmin_c, on_error = on_error, **kwargs)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('--tbase', 
        dest = 'tbase', 
        type = float, 
        default = 10.0, 
        help = 'The base temperature, in degrees Celsius.')
    parser.add_argument('--tmaxcutoff', 
        dest = 'tmaxcutoff', 
        type = float, 
        default = 999.0, 
        help = 'The maximum value allowed for the daily max air temperature')
    parser.add_argument('--tmincutoff', 
        dest = 'tmincutoff', 
        type = float, 
        default = -999.0, 
        help = 'The minimum value allowed for the daily min air temperature')
    parser.add_argument('tmax_c', 
        type = float, 
        help = 'The maximum daily air temperature in degrees Celsius.')
    parser.add_argument('tmin_c', 
        type = float, 
        help = 'The minimum daily air temperature in degrees Celsius.')

    # Parse the command line arguments
    args = parser.parse_args()

    # Call the run function.
    gdd = run(args.tmax_c, 
        args.tmin_c, 
        tbase = args.tbase, 
        tmaxcutoff = args.tmaxcutoff, 
        tmincutoff = args.tmincutoff)

    # Print the output to standard out.
    sys.stdout.write(str(gdd) + '\n')

    # Explicitly exit the script.
    sys.exit(0)


